import time
from SendToMicro import *

#Commands to call from other classes:
#--sendSimpleCommand
#--sendNewAlarmTimes
#--sendWindowName
#--sendCurrentTime
#--getLightSensorReadings
#--sendEraseAllWindowsCommand

class Commands:
    
    #-------------------------CONSTANTS-----------------------
    #For window time setting:
    WINDOW_TIMES_PER_WINDOW = 8
    WINDOW_TIMES_FILLER = 255
    #For sending simple commands:
    CMD_OPEN     = 0
    CMD_MIDDLE   = 1
    CMD_CLOSE    = 2
    CMD_GREEN    = 3
    CMD_SECURITY = 4
    
    #----------------------------TOOLS------------------------
        
    #"strTimeToBCD" aids in converting times (strings) to BCD
    def strTimeToBCD(self, strTime):
        if(len(strTime) < 2):
            strTime = '0' + strTime
        intTime_part1 = int(strTime[0])
        intTime_part2 = int(strTime[1])
        myBCD = 16*intTime_part1 + intTime_part2
        return myBCD
    
    def intTimeToBCD(self, intTime):
        intTime_part1 = int(intTime/10)
        intTime_part2 = int(intTime%10)
        return int(16*intTime_part1 + intTime_part2)
    
    #--------------------Category A Commands-------------------
    #"setup" returns an array of window name    
    def setup(self):
        stme = SendToMicroEfficient()
        stme.setup()
        c = ''
        c2 = ''
        windowNumbers = []
        windowNames = []
        print 'STARTING SETUP'
        #send a zero, pause, then a one
        stme.send(chr(0)) # 0 = listen for command
        x = stme.receive(1)
        print "x = " + str(ord(x))
        time.sleep(1.3)
#        time.sleep(3)
        stme.send(chr(254)) # 254 = start up
        y = stme.receive(1)
        print "y = " + str(ord(y))    
        print 'ready to receive windows...',
        for i in range(10): 
            print '.',
            c = stme.receive(1)
            print "c = " + str(ord(c))
            if(c == chr(255)):
                break        
            print '1',
            c2 = stme.receive(10)
            windowNumbers.append(c)
            windowNames.append(c2)
        #stme.receive(1) #final stop byte
        print 'FINISHED WITH SETUP'
        return windowNames
#        return windowNames, windowNumbers
    #----------------------------------------------------------

    #--------------------Category B Commands-------------------
    #"sendSimpleCommand" sends either open/middle/close/green/security
    def sendSimpleCommand(self, windowNumber, cmd_open=False, cmd_middle=False, cmd_close=False, cmd_green=False, cmd_security=False):
        mType = 'c'
        mWho = windowNumber
        mDataArr = [0,0,0,0,0,0,0,0,0,0]
        #Figure out which command:
        cmd = -1
        if(cmd_open == True):
            cmd = self.CMD_OPEN
        elif(cmd_middle == True):
            cmd = self.CMD_MIDDLE
        elif(cmd_close == True):
            cmd = self.CMD_CLOSE
        elif(cmd_green == True):
            cmd = self.CMD_GREEN
        elif(cmd_security == True):
            cmd = self.CMD_SECURITY
        if(cmd == -1):
            print "sendSimpleCommand error: no valid command sent"
            return
        mDataArr[0] = cmd
        self.send_generic_b_command(type=mType,who=mWho,dataArr=mDataArr)    
    
    #"sendNewAlarmTimes" sends all the alarm times for a window
    #times is an array (of length 8) of a 5-tuple of command, sec, min, hr, days
    #NOTE: sec, min, hr will be taken in normal, then converted to BCD
    #NOTE: IT REDIRECTS TO sendNewAlarmTimes2, THIS IS NO LONGER USED!
    def sendNewAlarmTimes(self, windowNumber, times):
        self.sendNewAlarmTimes2(windowNumber, times)
        if(0 == 1):        
            if len(times[0]) != 5:
                print "WRONG NUMBER OF TIMES! I WANT A TUPLE WITH 5 ELEMENTS"        
            #tell it that we are setting alarms:
            print "about to send generic b command...",
            self.send_generic_b_command(type='a', who=windowNumber)
            print "done."
            #set up communication class
            stme2 = SendToMicroEfficient()
            stme2.setup()
            #get a confirmation 'a':
            if(stme2.receive(1) != 'a'):
                print 'Warning: got unexpected alarm response'
            else:
                print 'Got expected alarm response'
            #########################################################
            #check object types
            for k in range(5):
                if (isinstance(times[0][k], int) == False):
                    print "WARNING: expected tuple element " + str(k) + " to be an int" 
            #########################################################    
            happycounter = 1 
            #tell it what alarms we are setting:
            for t in times:
                #Convert to BCD. open/close, sec, min, hr, day
                t2 = chr(t[0]), chr(self.intTimeToBCD(t[1])), chr(self.intTimeToBCD(t[2])), chr(self.intTimeToBCD(t[3])), chr(t[4])
                for tt in t2:
                    stme2.send(tt)
                    #////////////
                    print "send byte number " + str(happycounter)
                    happycounter += 1
                    #////////////
                    time.sleep(.2)
            #fill it up to 8:
            extraFillerTimes = self.WINDOW_TIMES_PER_WINDOW - len(times)
            for i in range(extraFillerTimes):
                for j in range(5):
                    stme2.send(chr(self.WINDOW_TIMES_FILLER))
                    #////////////
                    print "send FILLER byte number " + str(happycounter) + " that is equal to the byte value " + str(self.WINDOW_TIMES_FILLER)
                    happycounter += 1
                    #////////////
                    time.sleep(.2)
                
    def sendNewAlarmTimes2(self, windowNumber, times):
        # Pad "times" to be a 2D 8 x 5 matrix
        if len(times[0]) != 5:
            print "WRONG NUMBER OF TIMES! I WANT A TUPLE WITH 5 ELEMENTS"   
        while len(times) < 8:
            times.append((self.WINDOW_TIMES_FILLER, self.WINDOW_TIMES_FILLER, self.WINDOW_TIMES_FILLER, self.WINDOW_TIMES_FILLER, self.WINDOW_TIMES_FILLER))
        # Send the first command that we are sending an alarm
        print "starting..."
        print "type is " + str(type(times[0][0]))
        for n in range(4): #up to 4 because it sends two each time
            print "-----------------------------------------------------------"
            print "sending alarms " + str(2*n) + " and " + str(2*n + 1) + "...",
            print "BEFORE bcd...",
            print str(times[2*n])
            print "AND"
            print str(times[2*n+1])
            print "then after..."
            two_alarms = []
            if(times[2*n][0] != 255): #if it's not a filler
                two_alarms.append(times[2*n][0])                        # 1 open/close     do NOT convert
                two_alarms.append(self.intTimeToBCD(times[2*n][1]))     # 1 sec            convert to BCD
                two_alarms.append(self.intTimeToBCD(times[2*n][2]))     # 1 min            convert to BCD
                two_alarms.append(self.intTimeToBCD(times[2*n][3]))     # 1 hr             convert to BCD
                two_alarms.append(times[2*n][4])                        # 1 day            do NOT convert
            else: #if it IS a filler
                two_alarms.append(255)
                two_alarms.append(255)
                two_alarms.append(255)
                two_alarms.append(255)
                two_alarms.append(255)
            if(times[2*n+1][0] != 255):
                two_alarms.append(times[2*n+1][0])                      # 2 open/close     do NOT convert
                two_alarms.append(self.intTimeToBCD(times[2*n+1][1]))   # 2 sec            convert to BCD
                two_alarms.append(self.intTimeToBCD(times[2*n+1][2]))   # 2 min            convert to BCD
                two_alarms.append(self.intTimeToBCD(times[2*n+1][3]))   # 2 hr             convert to BCD
                two_alarms.append(times[2*n+1][4])                      # 2 day            do NOT convert
            else: #if it IS a filler
                two_alarms.append(255)
                two_alarms.append(255)
                two_alarms.append(255)
                two_alarms.append(255)
                two_alarms.append(255)
            print str(two_alarms) ###############
            c = '1'
            if(n == 1):
                c = '2'
            elif(n == 2):
                c = '3'
            elif(n == 3):
                c = '4'
            self.send_generic_b_command(type=c, who=windowNumber, dataArr=two_alarms, endConfirmation=True) #type will be 1,2,3,4 (which are all 'alarm' commands too)
            print "listening for response..."
        print "Done send alarms"
    
    #"sendWindowName"
    def sendWindowName(self, windowNumber, windowName):
        print "~~~~~~SENDING WINDOW NAME~~~~~~"
        #make windowName at least 10 characters long:
        windowName = windowName + "          "
        myDataArr = []
        for ii in range(10):
            myDataArr.append(ord(str(windowName)[ii]))        
        self.send_generic_b_command(type='n', who=windowNumber, dataArr=myDataArr)
        print "done sending name"
    
    #"sendCurrentTime" sends the current time to the head module
    def sendCurrentTime(self, waitAfter=True):
        print "~~~~~~~~~~~~~~ SENDING CURRENT TIME ~~~~~~~~~~~~~~~~~~~~~"
        lt  = time.localtime()
        sec_bcd  = self.strTimeToBCD(time.strftime('%S',lt))
        min_bcd  = self.strTimeToBCD(time.strftime('%M',lt))
        hour_bcd = self.strTimeToBCD(time.strftime('%H',lt))
        day = int(time.strftime('%w',lt)) + 1
        mType = 't' # 't' = time
        mWho  = 255 # 255 = all modules
        mDataArr = [0,0,0,0,0,0,0,0,0,0]
        mDataArr[0] = sec_bcd  # data[0] = sec
        mDataArr[1] = min_bcd  # data[1] = min
        mDataArr[2] = hour_bcd # data[2] = hr
        mDataArr[3] = day
        self.send_generic_b_command(type=mType,who=mWho,dataArr=mDataArr)
        ############ TODO: don't always wait here 
        if(waitAfter == True):
            time.sleep(3)
    
    #"getLightSensorReadings" returns light sensor 0, battery sensor 0, light sensor 1, battery sensor 1, etc....
    def getLightSensorReadings(self):
        self.send_generic_b_command(type='l',who=255)
        windowReadings = []
        stme = SendToMicroEfficient()
        stme.setup()
        x = -1
        while True:
            x = ord(stme.receive(1))
            if(x == 255):
                break
            else:
                windowReadings.append(x)
        print "light sensor readings = " + str(windowReadings)
        return windowReadings
        
    #"sendEraseAllWindowsCommand" erases all windows
    def sendEraseAllWindowsCommand(self):
        self.send_generic_b_command(type='e', who=255)
    
    #"send_generic_b_command" sends out a category b command
    # type must always be specified
    # who is the window # (by default all windows)
    # dataArr is the 10 bytes to send (by default all zeros)   
    def send_generic_b_command(self, type, who=255, dataArr=[0,0,0,0,0,0,0,0,0,0], endConfirmation=False):
        #format first 3 bytes
        byte_c = chr(0)
        byte_b = chr(who)
#        byte_b = chr(ord(who)) #int cast, in case passed as string
        ############################
        byte_a = type
#        if isinstance(type, int):
#            byte_a = chr(type)
#        else:
#            byte_a = type
        ############################
        #prep for send
        stme = SendToMicroEfficient()
        stme.setup()
        #Send Byte C / Get Confirmation
        stme.send(byte_c)
        stme.receive(1)
        time.sleep(1.3)
        #Send Byte B / Get Confirmation
        stme.send(byte_b)
        stme.receive(1)
        time.sleep(.3)
        #Send Byte A / NO Confirmation
        stme.send(byte_a)
        time.sleep(.3)
        #Send the 10 data bytes with delay / NO Confirmation
        for i in range(10):
            stme.send(chr(dataArr[i]))
            time.sleep(.2)
        if(endConfirmation == True):
            cc = stme.receive(1)
            if(cc == 'a'):
                print "Got positive confirmation of a"
            time.sleep(.2)
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
    #----------------------------------------------------------    
    #----------------------------------------------------------
    #----------------------------------------------------------